/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/cli/Command.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.cli;

import java.util.HashSet;
import java.util.Set;

/**
 * Represents a top level command and all of its available child commands.
 * A command is a case-sensitive string.
 * @author joshb
 *
 */
public class Command {
    /**
     * Constant definition of the default subCommand type (none).
     */
    public static final String NONE = "none";
    
    public static final String COMMAND_NAME_HELP = "help";
    public static final String COMMAND_NAME_VERSION = "version";
    
    /**
     * A default 'help' command. This command is commonly used across applications so
     * a default definition is provided.
     */
    public static final Command HELP = new Command(COMMAND_NAME_HELP);

    /**
     * A default 'version' command. This command is commonly used across applications so
     * a default definition is provided.
     */
    public static final Command VERSION = new Command(COMMAND_NAME_VERSION);

    private String name;
    private Set<Command> subCommands;

    public Command(String name) {
        setName(name);
        this.subCommands = new HashSet<Command>();
    }
    
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Set<Command> getSubCommands() {
        return this.subCommands;
    }

    public void addSubCommand(Command subCommand) {
        this.subCommands.add(subCommand);
    }
    
    @Override
    public int hashCode() {
        return this.name.hashCode();
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Command) {
            return this.name.equals(((Command) obj).name);
        }
        else {
            return false;
        }
    }
}
