/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/cli/binding/ObjectBinding.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.cli.binding;

import org.apache.commons.cli2.Argument;
import org.apache.commons.cli2.CommandLine;
import org.apache.commons.cli2.Option;
import org.apache.commons.cli2.option.DefaultOption;

import com.netapp.common.cli.CliParameter;
import com.netapp.common.cli.validator.ValidationException;

/**
 * A binding between a the ApacheCLI option and the Class that option corresponds to
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public abstract class ObjectBinding {
    private CliParameter cliParameter;
    private Option option;
    private Class<?> objectClass;
    private boolean parentIsArray;
    
    protected ObjectBinding(Class<?> objectClass, CliParameter cliParameter) {
        this.objectClass = objectClass;
        this.cliParameter = cliParameter;
    }
    
    protected ObjectBinding(Class<?> objectClass, CliParameter cliParameter, Option option) {
        this(objectClass, cliParameter);
        this.option = option;
    }

    protected Class<?> getObjectClass() {
        return this.objectClass;
    }
    
    public void setOption(Option option) {
        this.option = option;
    }
    
    public Option getOption() {
        return this.option;
    }
    
    public CliParameter getCliParameter() {
        return this.cliParameter;
    }
    
    public final Object hydrate(CommandLine commandLine) throws ValidationException {
        this.validate(commandLine);
        return this.doHydrate(commandLine);
    }
    
    public String getOptionDisplay() {
        String optionName = "-" + this.cliParameter.option();
        Argument argument = ((DefaultOption) this.option).getArgument();
        if (argument != null) {
            optionName = optionName + " <" + argument.getPreferredName() + ">";
        }
        return optionName;
    }
    
    /**
     * Validate that this object is bound correctly on the CommandLine
     */
    protected abstract void validate(CommandLine commandLine) throws ValidationException;

    /**
     * Construct an instance of the object from the CommandLine
     */
    public abstract Object doHydrate(CommandLine commandLine) throws ValidationException;
    
    public boolean isNull(CommandLine commandLine) {
        return !commandLine.hasOption(this.option);
    }
    
    /**
     * Expose if the object is an array without exposing the object class.
     * 
     * @return true if this binding is an array type
     */
    public boolean isObjectAnArray() {
        return getObjectClass().isArray();
    }
    
    public boolean isParentIsArray() {
        return this.parentIsArray;
    }

    public void setParentIsArray(boolean parentIsArray) {
        this.parentIsArray = parentIsArray;
    }
}