/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/cli/validator/RegexValidator.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.cli.validator;

import java.util.List;
import java.util.regex.Pattern;

import org.apache.commons.cli2.CommandLine;

import com.netapp.common.MsgKey;
import com.netapp.common.cli.binding.ArrayObjectBinding;
import com.netapp.common.cli.binding.BeanObjectBinding;
import com.netapp.common.cli.binding.PrimitiveObjectBinding;
import com.netapp.common.logging.MessageBundle;
import com.netapp.common.logging.MessageKey;

/**
 * Validates bound CliParameter against a regular expression
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class RegexValidator implements Validator {
    private Pattern pattern;

    private MessageKey msgKey;

    public RegexValidator(Pattern pattern) {
        this(pattern, MsgKey.CLI_BASE_PARAMETER_INVALID);
    }
    
    public RegexValidator(Pattern pattern, MessageKey msgKey) {
        this.pattern = pattern;
        this.msgKey = msgKey;
    }

    @SuppressWarnings("unchecked") // List<String> cast from Apache CLI
    public void validate(PrimitiveObjectBinding primitiveObjectBinding, CommandLine commandLine)
        throws ValidationException {
        if (primitiveObjectBinding.isParentIsArray()) {
            List<String> values = commandLine.getValues(primitiveObjectBinding.getOption());
            
            for (String value : values) {
                if (value != null && !this.pattern.matcher(value).matches()) {
                    throw new ValidationException(MessageBundle.localizeMessage(msgKey, new Object[] { value, primitiveObjectBinding.getOptionDisplay() }));
                }
            }
        }
        else {
            String value = primitiveObjectBinding.getRawValue(commandLine);
            if (value != null && !this.pattern.matcher(value).matches()) {
                throw new ValidationException(MessageBundle.localizeMessage(msgKey, new Object[] { value, primitiveObjectBinding.getOptionDisplay() }));
            }
        }
    }

    public void validate(BeanObjectBinding beanObjectBinding, CommandLine commandLine) throws ValidationException {
        assert false : "Shouldn't be validating a regex against a bean";
    }
    
    public void validate(ArrayObjectBinding collectionObjectBinding, CommandLine commandLine) throws ValidationException {
        assert false : "Shouldn't be validating a regex against a collection";
    }

}
