/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/db/derby/EmbeddedDerbyDatabase.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.db.derby;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.apache.derby.jdbc.EmbeddedDataSource;

public class EmbeddedDerbyDatabase {
    private String storageLocation;
    private Driver databaseDriver;
    
    public EmbeddedDerbyDatabase() {
        
    }
    
    public EmbeddedDerbyDatabase(String storageLocation) {
        this.storageLocation = storageLocation;
    }
    
    public void setStorageLocation(String storageLocation) {
        this.storageLocation = storageLocation;
    }
    
    public synchronized void startDatabase() throws SQLException {
        if (this.databaseDriver != null) {
            return;
        }

        System.setProperty("derby.system.home", this.storageLocation);
                
        // load driver
        try {
            this.databaseDriver = (Driver)Class.forName("org.apache.derby.jdbc.EmbeddedDriver").newInstance();
        }
        catch (ClassNotFoundException e) {
            throw new SQLException(e.getLocalizedMessage());
        }
        catch (InstantiationException e) {
            throw new SQLException(e.getLocalizedMessage());
        }
        catch (IllegalAccessException e) {
            throw new SQLException(e.getLocalizedMessage());
        }
    }
    
    public synchronized boolean shutdown() {
        boolean shutdown = false;

        try {
            if (this.databaseDriver != null) {
                this.databaseDriver.connect("jdbc:derby:crashdb;shutdown=true", null);
                this.databaseDriver = null;
            }
        }
        catch (SQLException se) {
            // ignore errors, Derby throws an error even on clean shutdown
        }

        return shutdown;
    }
    
    public DataSource getDataSource(DerbyConnectionProperties properties) {
        EmbeddedDataSource dataSource = new EmbeddedDataSource();
    
        dataSource.setCreateDatabase(String.valueOf(properties.getCreateDatabase()));
        dataSource.setUser(properties.getUser());
        dataSource.setPassword(properties.getPassword());
        dataSource.setDatabaseName(properties.getDatabaseName());
        
        return dataSource;
    }
    
    public Connection getConnection(DerbyConnectionProperties properties) throws SQLException {
        StringBuffer connectionString = new StringBuffer("jdbc:derby:");
        connectionString.append(properties.getDatabaseName());
        
        if (properties.getCreateDatabase()) {
            connectionString.append(";create=true");
        }
        
        return DriverManager.getConnection(connectionString.toString());
    }

    @Override
    protected void finalize() throws Throwable {
        shutdown();
        super.finalize();
    }
    
    
}
