/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/encryption/DES.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.encryption;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;

import org.apache.commons.codec.binary.Base64;

/**
 * Utility class to handle setting up javax.crypto to do DES encryption.
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DES implements ByteArrayCipher, StringCipher, StreamCipher {
    private static final String CHAR_ENCODING = "UTF-8";
    
    private static final byte[] SALT = {
        (byte) 0xfe, (byte) 0xdc, (byte) 0xba, (byte) 0x98, 
        (byte) 0x76, (byte) 0x54, (byte) 0x32, (byte) 0x10
    };
    
    private static final int ITERATION_COUNT = 17;
    
    private final String algorithm;
    private final SecretKey key;
    private final AlgorithmParameterSpec params;
    private transient Cipher eCipher;
    private transient Cipher dCipher;
    
    public DES(String algorithm, SecretKey key, AlgorithmParameterSpec params) {
        this.algorithm = algorithm;
        this.key = key;
        this.params = params;
    }
    
    /**
     * Construct a new DES object using the supplied password to generate the
     * DES key.
     * @param password Plaintext password
     */
    public DES(String password) {
        try {
            KeySpec spec = new PBEKeySpec(password.toCharArray(), SALT, ITERATION_COUNT);
            this.key = SecretKeyFactory.getInstance("PBEWithMD5AndDES").generateSecret(spec);
            this.algorithm = this.key.getAlgorithm();
            this.params = new PBEParameterSpec(SALT, ITERATION_COUNT);
        }
        catch (InvalidKeySpecException e) {
            throw new RuntimeException(e);
        }
        catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }   
    }

    /**
     * @see com.netapp.common.encryption.ByteArrayCipher#encrypt(byte[])
     */
    public byte[] encrypt(byte[] input) {
        try {
            return this.getEncryptionCipher().doFinal(input);
        }
        // These cannot happen if we setup the cipher correctly
        catch (IllegalBlockSizeException e) {
            throw new RuntimeException(e);
        }
        catch (BadPaddingException e) {
            throw new RuntimeException(e);
        }
    }
    
    /**
     * @see com.netapp.common.encryption.ByteArrayCipher#decrypt(byte[])
     */
    public byte[] decrypt(byte[] input) {       
        try {
            return this.getDecryptionCipher().doFinal(input);
        }
        catch (IllegalBlockSizeException e) {
            throw new RuntimeException(e);
        }
        catch (BadPaddingException e) {
            throw new RuntimeException(e);
        }
    }
    
    /**
     * @see com.netapp.common.encryption.StringCipher#encrypt(java.lang.String)
     */
    public String encrypt(String plaintext) {
        try {
            byte[] bytes = plaintext.getBytes(CHAR_ENCODING);
            bytes = encrypt(bytes);
            return new String(Base64.encodeBase64(bytes), CHAR_ENCODING);
        }
        catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }
    
    /**
     * @see com.netapp.common.encryption.StringCipher#decrypt(java.lang.String)
     */
    public String decrypt(String encrypted) {
        try {
            byte[] bytes = Base64.decodeBase64(encrypted.getBytes(CHAR_ENCODING));
            bytes = decrypt(bytes);
            return new String(bytes, CHAR_ENCODING);
        }
        catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }
    /**
     * 
     * @see com.netapp.common.encryption.StreamCipher#decrypt(java.io.InputStream)
     */
    public InputStream decrypt(InputStream encrypted) {
        BufferedInputStream stream = new BufferedInputStream(encrypted);
        return new CipherInputStream(stream, getDecryptionCipher());
    }
    
    /**
     * 
     * @see com.netapp.common.encryption.StreamCipher#encrypt(java.io.OutputStream)
     */
    public OutputStream encrypt(OutputStream plaintext) {
        return new CipherOutputStream(plaintext, getEncryptionCipher());
    }
    
    private Cipher getEncryptionCipher() {
        if (this.eCipher == null) {
            this.eCipher = createCipher(Cipher.ENCRYPT_MODE);
        }
        return this.eCipher;
    }
    
    private Cipher getDecryptionCipher() {
        if (this.dCipher == null) {
            this.dCipher = createCipher(Cipher.DECRYPT_MODE);
        }
        return this.dCipher;
    }
    
    private Cipher createCipher(int opmode) {
        try {
            Cipher cipher = Cipher.getInstance(this.algorithm);
            cipher.init(opmode, this.key, this.params);
            return cipher;
        }
        // These should not happen since we've setup the cipher correctly
        catch (InvalidKeyException e) {
            throw new RuntimeException(e);
        }
        catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
        catch (NoSuchPaddingException e) {
            throw new RuntimeException(e);
        }
        catch (InvalidAlgorithmParameterException e) {
            throw new RuntimeException(e);
        }
    }
    
    /**
     * Construct a new DES object using the default password.
     * This is only useful for obfuscating things.
     *
     * @return Default application cipher
     */
    public static StringCipher getApplicationCipher() {
        return new DES("JulQbVObgure");
    }
    
    /**
     * Get the cipher used by SDU.
     * @return SDU cipher
     */
    @SuppressWarnings("unused")
    public static DES getSnapdriveUnixCipher() {    
        try {
            final String cipherName = "DES";
            final String cipherOptions = "/PCBC/NoPadding";

            final AlgorithmParameterSpec params = new IvParameterSpec(SALT);
            final SecretKey key = SecretKeyFactory.getInstance(cipherName).generateSecret(
                new DESKeySpec(new byte[] {
                    (byte) 0xdc, (byte) 0xfd, (byte) 0xdf, (byte) 0x75,
                    (byte) 0xcd, (byte) 0xae, (byte) 0xab, (byte) 0x8a}));
            return new DES(cipherName + cipherOptions, key, params);
        }
        catch (InvalidKeyException e) {
            throw new RuntimeException(e);
        }
        catch (InvalidKeySpecException e) {
            throw new RuntimeException(e);
        }
        catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        } 
    }
    
    public static void main(String[] args) throws UnsupportedEncodingException {
        if (args.length != 1) {
            System.err.println("Usage: DES <secret>");
            System.exit(1);
        }
        StringCipher des = getApplicationCipher();
        String a = des.encrypt(args[0]);
        System.out.println("{des}" + a);
    }
}
