/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/execution/ShellExecutor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.execution;

import java.io.File;
import java.util.List;

/**
 * A class capable of executing commands via an operating system shell.
 * Takes in structured {@link ShellCommand} or {@link ShellRunnable} objects 
 * and returns structured {@link ShellResult} objects 
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public interface ShellExecutor {

    /**
     * Execute a shell command interactively
     * @return a handle to the raw {@link ProcessContext} object
     */
    ProcessContext executeInteractive(ShellRunnable shellCommand);
    
    /**
     * Execute a shell command but does not wait for the shell command to complete.
     * The call should invoke one of the waitFor methods on ShellResult to wait
     * for the shell command to complete. This is useful when the output is required
     * even if the shell command is terminated early due to a timeout.
     * 
     * @param shellCommand
     * @return
     */
    ShellResult executeAsync(ShellRunnable shellCommand);
    
    /**
     * Execute the shell command 
     * Blocks until the command is complete
     * @return A structured {@link ShellResult} object encapsulating STDOUT, STDERR, exit code, etc.
     */
    ShellResult execute(ShellRunnable shellCommand);

    /**
     * Execute the shell command
     * Blocks until the command is complete or {@code timeoutInMsec} seconds have elapsed
     * @param timeoutInMsec Duration to wait before killing the process and returning
     * @return A structured {@link ShellResult} object encapsulating STDOUT, STDERR, exit code, etc.
     */
    ShellResult execute(ShellRunnable shellCommand, long timeoutInMsec);
    
    void setEnvironmentVariable(EnvironmentVariable environmentVariable);
    
    void unsetEnvironmentVariable(String key);
    
    EnvironmentVariable getEnvironmentVariable(String key);

    List<EnvironmentVariable> getUserRelevantEnvironmentVariables();
    
    /**
     * Sets the working directory for the command that is going to be executed.  This value may be 
     * overridden by each ShellCommand.
     * @param directory The working directory.
     */
    void setWorkingDirectory(File directory);
    File getWorkingDirectory();
    
    String getUser();
}
