/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/execution/UnixUserSwitchingShellExecutor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.execution;

import java.util.Arrays;
import java.util.List;

import com.netapp.common.platform.OperatingSystem;

class UnixUserSwitchingShellExecutor extends AbstractShellExecutor {

    private static final String APPLICATION_LIBRARY_PATH = "APP_LIBRARY_PATH";

    protected String user;
    protected ShellCommand commandExecuted;

    UnixUserSwitchingShellExecutor(ShellCommandTransformer shellCommandTransformer, String user) {
        super(shellCommandTransformer);
        this.user = user;
        
        /*
         * See burt 244939
         * If we don't unset this, bash will try to read root's .bashrc file as the su'ed user
         * which will cause a permission denied error 
         */
        this.unsetEnvironmentVariable("BASH_ENV");
    }
   
    public ProcessContext executeInteractive(ShellRunnable shellCommand) {

        setApplicationLibraryPath();
        
        return this.doExecute(shellCommand);
        
    }

    private void setApplicationLibraryPath() {

        /*
         * When su is executed on Solaris, it will strip off the 
         * LD_LIBRARY_PATH environment variable (see burt 229852)
         * Therefore, we copy the LD_LIBRARY_PATH to APP_LIBRARY_PATH.
         * Then the command that is executed can append the
         * APP_LIBRARY_PATH back onto the LD_LIBRARY_PATH if necessary. 
         */
        
        EnvironmentVariable libraryPathVariable = getEnvironmentVariable(OperatingSystem.createInstance().getLibraryEnvironmentName());
        if (libraryPathVariable.getValue() != null) {
            setEnvironmentVariable(new EnvironmentVariable(APPLICATION_LIBRARY_PATH, libraryPathVariable.getValue()));
        }
        
    }

    @Override
    public List<EnvironmentVariable> getUserRelevantEnvironmentVariables() {
        return Arrays.asList(new EnvironmentVariable("USER", this.user));
    }
    
    @Override
    public String getUser() {
        return this.user;
    }
}
