/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/execution/WindowsShellExecutor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.execution;

import java.io.File;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;


class WindowsShellExecutor extends DefaultShellExecutor {

    WindowsShellExecutor(ShellCommandTransformer shellCommandTransformer) {
        super(shellCommandTransformer);
        String windowsRoot = System.getenv("SystemRoot");
        this.setEnvironmentVariable(new EnvironmentVariable("PATH",  windowsRoot + File.separator + "system32" 
            + File.pathSeparator + this.getEnvironmentVariable("PATH").getValue()));
    }
    
    /*
     * See burt 251610 and burt 251693
     * 
     * http://wrapper.tanukisoftware.org/doc/english/props-envvars.html
     * "When the Wrapper is run as a service, environment variables will be loaded from the system
     * registry rather than from the environment. This was necessary because Windows loads the environment
     * variables which are available to services when the machine is booted. Any changes to the system environment
     * variables in the registry (set directly or through the system control panel) are not made available
     * to the services until the machine is once again rebooted. By loading the environment variables from the
     * registry, the reboot can be avoided while providing the same functionality."
     * 
     * I have filed a bug 1738437 against the "java service wrapper" for this behavior
     * http://sourceforge.net/tracker/index.php?func=detail&aid=1738437&group_id=39428&atid=425187
     * 
     * This class must work in "directly launched" environments as well as environments launched 
     * via the "java service wrapper".  So we make sure to deal with the case-insensitivity of Windows
     * environment variables below.
     */
    
    @Override
    public EnvironmentVariable getEnvironmentVariable(String key) {
        /*
         * Environment variables on Windows are NOT case sensitive
         * Return the first environment variable that ignore-case-equals the key 
         */
        String value = null;
        for (Map.Entry<String, String> entry : this.getProcessBuilder().environment().entrySet()) {
            String environmentKey = entry.getKey();
            String environmentValue = entry.getValue();
            if (key.equalsIgnoreCase(environmentKey)) {
                value = environmentValue;
                break;
            }
        }
        
        return new EnvironmentVariable(key, value);
    }
    
    @Override
    public void setEnvironmentVariable(EnvironmentVariable environmentVariable) {
        /*
         * Environment variables on Windows are NOT case sensitive
         * Unset all other environment variables that ignore-case-equal the key 
         */
        String key = environmentVariable.getKey();

        Set<String> keysToRemove = new HashSet<String>();
        for (String environmentKey : this.getProcessBuilder().environment().keySet()) {
            if (key.equalsIgnoreCase(environmentKey)) {
                keysToRemove.add(environmentKey);
            }
        }
        for (String keyToRemove : keysToRemove) {
            this.unsetEnvironmentVariable(keyToRemove);    
        }
        
        /*
         * Now uppercase the key and set it in the environment
         */
        super.setEnvironmentVariable(new EnvironmentVariable(key.toUpperCase(), environmentVariable.getValue()));
    }

}
