/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/FlowConstants.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

import static com.netapp.common.util.ConfigProperty.intProperty;
import static com.netapp.common.util.ConfigProperty.longProperty;
import static com.netapp.common.util.ConfigProperty.booleanProperty;
import static com.netapp.common.util.ConfigProperty.fileProperty;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

import com.netapp.common.util.ConfigProperty;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class FlowConstants {
    private FlowConstants() { }

    public static final String OPERATION_MDC = "operation";
    public static final String CLIENT_ID_MDC = "clientRequestId";
    public static final String OPERATION_TYPE_MDC = "operationType";

    public static final Map<String, Class<?>> SHORT_TYPES = new HashMap<String, Class<?>>();
    static {
        SHORT_TYPES.put("string", String.class);
        SHORT_TYPES.put("integer", Integer.class);
        SHORT_TYPES.put("boolean", Boolean.class);
    }

    public static final Pattern OPERATION_NAME_REGEX = Pattern.compile("[a-zA-Z][a-zA-Z0-9_\\-]*");
    
    public static final String WORKFLOW_SCHEMA_NAMESPACE = "http://www.netapp.com/common/flow/workflow";  
    public static final String WORKFLOW_SCHEMA_URI = "http://www.netapp.com/common/flow/workflow.xsd";

    /**
     * Classpath location for {@link com.netapp.common.xml.CatalogEntityResolver CatalogEntityResolver}
     * catalog files required to parse schemas. All implementations must put a catalog file there
     * containing the extended schemas for that project.
     */
    public static final String WORKFLOW_CATALOG_PATH = "catalog/workflow-schemas.xml";
    public static final String ABORT_POINT_ATTRIBUTE = "abortPoint";
    public static final String CLASSNAME_ATTRIBUTE = "classname";
    public static final String FAILURE_STRATEGY_ATTRIBUTE = "failureStrategy";
    public static final String IF_CONDITION_ATTRIBUTE = "if";
    public static final String UNLESS_CONDITION_ATTRIBUTE = "unless";
    public static final String NAME_ATTRIBUTE = "name";
    public static final String REGISTER_ATTRIBUTE = "register";
    public static final String APPEND_ATTRIBUTE = "append";
    public static final String THREADPOOL_ATTRIBUTE = "threadpool";
    public static final String TYPE_ATTRIBUTE = "type";
    public static final String VALUES_ATTRIBUTE = "values";
    public static final String VAR_ATTRIBUTE = "var";
    
    public static final String ABORT_TASKS_ELEMENT = "AbortTasks";
    public static final String CHECKPOINT_ELEMENT = "Checkpoint";
    public static final String DIAGNOSTIC_ELEMENT = "Diagnostic";
    public static final String FOR_ELEMENT = "For";
    public static final String INPUT_ELEMENT = "Input";
    public static final String OPERATION_ELEMENT = "Operation";
    public static final String OUTPUT_ELEMENT = "Output";
    public static final String REGISTER_ELEMENT = "Register";
    public static final String TASKS_ELEMENT = "Tasks";
    public static final String TYPE_ELEMENT = "Type";
    
    public static final int DEFAULT_THREADPOOL_SIZE = 10;
    public static final int DEFAULT_THREADPOOL_IDLE = 5;

    public static final String DIAGNOSTIC_OPERATION_NAME = "_diagnostic";
    
    public static final String FAILED_OPERATION_ID = "failedOperationId";
    public static final String FAILED_OPERATION_CLIENT_REQUEST_ID = "failedOperationClientRequestId";
    public static final String FAILED_OPERATION_TYPE = "failedOperationType";
    public static final String FAILED_OPERATION_REGISTERS = "failedOperationRegisters";
    
    /** 
     * The lifetime of a context after it is in a terminal state in minutes. 
     * The default if not specified is up to the individual manager. 
     * */
    public static final ConfigProperty<Long> CONTEXT_LIFETIME = longProperty("flow.persistence.contextLifetime");
    /**
     * The interval between context scavenges, in minutes.
     * The default if not specified is up to the individual manager.
     */
    public static final ConfigProperty<Long> CONTEXT_SCAVENGE_INTERVAL = longProperty("flow.persistence.scavengeInterval");
    
    /**
     * Whether to scavenge INCOMPLETE operations or not.
     */
    public static final ConfigProperty<Boolean> CONTEXT_SCAVENGE_INCOMPLETE = booleanProperty("flow.persistence.scavengeIncomplete");
    
    /**
     * Whether to automatically requeue running operations on restart.
     */
    public static final ConfigProperty<Boolean> CONTEXT_REQUEUE_ON_INIT = booleanProperty("flow.persistence.requeueOnInit");

    /**
     * Whether to encrypt sensitive information in the persistent store.
     * Note: Changing this option may invalidate currently saved contexts.
     */
    public static final ConfigProperty<Boolean> PERSISTENCE_ENCRYPTION_ENABLE = booleanProperty("flow.persistence.encryption.enable", false);
    
    /**
     * How many times to retry to get a connection if the database is down.
     * This is currently only tested or recommended with '-1' to wait forever. Other settings may cause exceptions in unexpected places
     * or loss of data. Waiting forever ensures we never pass a checkpoint and continue unless it gets recorded. 
     */
    public static final ConfigProperty<Integer> PERSISTENCE_CONNECTION_RETRY_COUNT = intProperty("flow.persistence.connection.retry.count");
    
    /**
     * How many times to retry to get a connection if the database is down for a log operation.
     * Once this is exceeded, the log will be dropped but the operation continue.
     */
    public static final ConfigProperty<Integer> PERSISTENCE_CONNECTION_LOG_RETRY_COUNT = intProperty("flow.persistence.connection.log.retry.count");
    
    /**
     * How long to wait between successive connection attempts.
     */
    public static final ConfigProperty<Integer> PERSISTENCE_CONNECTION_RETRY_IDLE = intProperty("flow.persistence.connection.retry.idle");
    
    /**
     * Whether to automatically run diagnostics on a failed operation.
     */
    public static final ConfigProperty<Boolean> DIAGNOSTIC_AUTO_ENABLE = booleanProperty("flow.diagnostic.auto.enable", false);

    /**
     * Where to place the diag zip files
     */
    public static final ConfigProperty<File> DIAGNOSTIC_OUTPUT_PATH = fileProperty("flow.diagnostic.output.path", new File("diagnostics"));

    /**
     * Where to place the embedded database files
     */
    public static final ConfigProperty<File> PERSISTENCE_EMBEDDED_STORAGE_LOCATION = fileProperty("flow.persistence.embedded.storageLocation", new File("crash"));

    public static final ConfigProperty<Boolean> ABORT_DEBUG_DISABLE = booleanProperty("flow.debug.disableAbort", false);
}
