/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/soap/ServiceImplFactory.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow.soap;

import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.HashMap;
import java.util.Map;

import javax.jws.WebMethod;
import javax.jws.WebParam;

import com.netapp.common.MsgKey;
import com.netapp.common.flow.Engine;
import com.netapp.common.flow.EngineConfiguration;
import com.netapp.common.flow.FlowConstants;
import com.netapp.common.flow.FlowException;
import com.netapp.common.flow.OperationId;
import com.netapp.common.flow.PersistenceManager;
import com.netapp.common.flow.api.OperationResult;
import com.netapp.common.flow.api.OperationService;
import com.netapp.common.flow.api.OperationStartResult;
import com.netapp.common.logging.NALogger;
import com.netapp.common.util.ConfigBundle;
import com.netapp.common.util.StringUtils;

/**
 * Construct a proxy object to connect a WebService interface to
 * a workflow engine. The generated object will handle the following
 * WebMethod interactions:
 * <ul>
 * <li>Any operations defined in {@link OperationService}, if the supplied interface extends from OperationService
 * <li>Any web method of the signature <samp>&lt;T extends {@link OperationResult}&gt; T operationNameResult(String operationId)</samp>
 * by constructing a new <samp>T</samp> object, populating any additional fields with 
 * the contents of the operation registers upon completion.
 * <li>Any web method of the signature <samp>{@link OperationStartResult} operationName(? ...)</samp> by starting a new operation
 * of the same name with the supplied parameters. The method must have an appropriate {@literal @WebMethod} annotation and each parameter
 * must have a {@literal @WebParam} annotation declaring the name defined in the workflow descriptor.
 * </ul>
 * 
 * @deprecated {@link WorkflowBindingFactory} is now the preferred way of binding workflows to Java methods.
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
@Deprecated
public class ServiceImplFactory {
    private static final NALogger logger = NALogger.getLogger(ServiceImplFactory.class);
    
    private ConfigBundle configBundle;
    private PersistenceManager persistenceManager;
    private ResultDelegate resultDelegate;
    private Map<String, Object> serviceHandlers = new HashMap<String, Object>();
   
    public ServiceImplFactory() {
    }

    public void setServiceHandlers(Map<String, Object> serviceHandlers) {
        this.serviceHandlers = serviceHandlers;
    }
    
    public void setApplicationContext(ConfigBundle configBundle) {
        this.configBundle = configBundle;
    }
    
    public void setPersistenceManager(PersistenceManager persistenceManager) {
        this.persistenceManager = persistenceManager;
    }
    
    public void setResultDelegate(ResultDelegate resultDelegate) {
        this.resultDelegate = resultDelegate;
    }
    
    /**
     * Construct a new ServiceImpl class for the specified WebService interface.
     * Any WebMethods in the specified interface will be mapped as defined in the
     * class description.
     * @param clazz Service interface class
     * @return Service implementation for the supplied interface.
     */
    public Object createServiceImpl(Class<?> clazz) {
        assert this.persistenceManager != null;

        if (!clazz.isInterface()) {
            throw new FlowException(logger.error(MsgKey.FLOW_SOAP_SERVICE_NOT_INTERFACE, clazz.getName()));
        }
        
        EngineConfiguration configuration = new EngineConfiguration();
        WorkflowDiagnostic diag = clazz.getAnnotation(WorkflowDiagnostic.class);
        if (diag != null) {
            configuration.addOperation(FlowConstants.DIAGNOSTIC_OPERATION_NAME, diag.value());
        }
        
        for (Method m : clazz.getMethods()) {
            if (m.getReturnType() != OperationStartResult.class) {
                continue;
            }
            
            WorkflowOperation op = m.getAnnotation(WorkflowOperation.class);
            if (op == null) {
                throw new FlowException(logger.error(MsgKey.FLOW_SOAP_SERVICE_NOT_WORKFLOW_OPERATION, m));
            }
            configuration.addOperation(getWebMethodName(m), op.value());
        }
        Engine engine = new Engine();
        engine.setPersistenceManager(this.persistenceManager);
        engine.setApplicationContext(this.configBundle);
        engine.setConfiguration(configuration);
        engine.start();
        ServiceInvocationHandler handler = new ServiceInvocationHandler(engine, this.serviceHandlers);
        handler.setResultDelegate(this.resultDelegate);
        
        return Proxy.newProxyInstance(clazz.getClassLoader(), new Class<?>[] {clazz}, handler);
    }
    
    public Object createServiceImpl(String className) {
        try {
            Class<?> clazz = Class.forName(className);
            return createServiceImpl(clazz);
        }
        catch (ClassNotFoundException e) {
            throw new FlowException(logger.error(MsgKey.FLOW_SOAP_SERVICE_CLASS_NOT_FOUND, className));
        }
    }
    
    static String getWebMethodName(Method method) {
        String operationName;
        WebMethod webMethod = method.getAnnotation(WebMethod.class);
        if (webMethod != null && !StringUtils.isEmpty(webMethod.operationName())) {
            operationName = webMethod.operationName();
        }
        else {
            operationName = method.getName();
        }
        return operationName;
    }
    
    private static class ServiceInvocationHandler implements InvocationHandler {
        private Engine engine;
        private OperationServiceImpl operationService;
        private Map<String, Object> serviceHandlers;
        
        public ServiceInvocationHandler(Engine engine, Map<String, Object> serviceHandlers) {
            this.engine = engine;
            this.operationService = new OperationServiceImpl(this.engine);
            this.serviceHandlers = serviceHandlers;
        }
        
        protected void setResultDelegate(ResultDelegate resultDelegate) {
            this.operationService.setResultDelegate(resultDelegate);
        }
        
        public Object invoke(Object obj, Method method, Object[] paramArray) throws Throwable {
            Object delegateHandler = lookupHandler(method);
            
            if (delegateHandler != null) {
                return method.invoke(delegateHandler, paramArray);
            }
            
            // Custom result object
            if (method.getName().endsWith("Result")
                    && OperationResult.class.isAssignableFrom(method.getReturnType())
                    && paramArray.length == 1) {
                return doResult(obj, method, paramArray);  
            }
            
            // Otherwise it's a OperationStart call, so put a request together for the
            // workflow engine.
            OperationStartResult startResult = new OperationStartResult();
            try {
                startResult.setId(doStart(obj, method, paramArray).getId());
            }
            // CHECKSTYLE:OFF
            catch (Throwable t) {
            // CHECKSTYLE:ON
                startResult.setError(t.getLocalizedMessage());
            }
            return startResult;
        }

        private Object lookupHandler(Method method) {
            // If the method was in the OperationService interface, delegate it to
            // a handler for that
            if (method.getDeclaringClass().equals(OperationService.class)) {
                return this.operationService;
            }
            
            for (String interfaceDefinition : this.serviceHandlers.keySet()) {
                try {
                    Class<?> interfaceClass = Class.forName(interfaceDefinition);
                    
                    if (method.getDeclaringClass().equals(interfaceClass)) {
                        return this.serviceHandlers.get(interfaceDefinition);
                    }
                }
                catch (ClassNotFoundException e) {
                    // ignore, the missing class may have nothing to do with the method call
                }
            }
            
            return null;
        }
        
        @SuppressWarnings("unchecked") // Class cast is unchecked
        private OperationResult doResult(Object obj, Method method, Object[] paramArray) throws Throwable {
            Class<? extends OperationResult> clazz = (Class<OperationResult>)method.getReturnType();
            return this.operationService.operationResult(paramArray[0].toString(), clazz);
        }
        
        private OperationId doStart(Object obj, Method method, Object[] paramArray) throws Throwable {
            String operationName = null;
            if (!OperationStartResult.class.equals(method.getReturnType())) {
                throw new FlowException(logger.error(MsgKey.FLOW_SOAP_SERVICE_INVALID_METHOD_RETURN, method.toString()));
            }
            
            operationName = getWebMethodName(method);
            if (StringUtils.isEmpty(operationName)) { 
                throw new FlowException(logger.error(MsgKey.FLOW_SOAP_SERVICE_NOT_WEB_METHOD, method.toString()));
            }

            Annotation[][] annotations = method.getParameterAnnotations();
            Map<String, Object> params = new HashMap<String, Object>();
        
        param: 
            for (int i = 0; i < annotations.length; i++) {
                for (Annotation a : annotations[i]) {
                    if (a instanceof WebParam) {
                        WebParam wpa = (WebParam)a;
                        params.put(wpa.name(), paramArray[i]);
                        continue param;
                    }
                    throw new FlowException(logger.error(MsgKey.FLOW_SOAP_SERVICE_MISSING_PARAMETER, method.toString(), i, WebParam.class));
                }
            }
            return this.engine.operationStart(operationName, params);
        }
    }

}
