/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/gwt/codegen/GWTInterfaceConverter.java#1 $
 * 
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.gwt.codegen;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.VariableElement;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.util.ElementFilter;
import javax.lang.model.util.Elements;
import javax.lang.model.util.Types;
import javax.xml.bind.annotation.XmlTransient;

import com.netapp.common.generator.NetappClassGenerator;
import com.netapp.common.util.StringUtils;

public class GWTInterfaceConverter extends NetappClassGenerator {

    private TypeElement typeElement;
    private Elements elements;
    private Types types;
    private boolean isService;

    public GWTInterfaceConverter(TypeElement typeElement, Elements elements, Types types) {
        this(typeElement, elements, types, true);
    }

    public GWTInterfaceConverter(TypeElement typeElement, Elements elements, Types types, boolean isService) {
        super(elements.getPackageOf(typeElement).getQualifiedName().toString());
        
        this.typeElement = typeElement;
        this.elements = elements;
        this.types = types;
        this.isService = isService;
        
        generateClassFile();
    }
    
    private void generateClassFile() {
        String interfaceName = this.typeElement.getSimpleName().toString();
    
        List<String> extendedInterfaces = new LinkedList<String>();
        for (TypeMirror extendedInterfaceType : this.typeElement.getInterfaces()) {
            extendedInterfaces.add(extendedInterfaceType.toString());
        }
//        
//        if (this.isService) {
//            extendedInterfaces.add("com.google.gwt.user.client.rpc.RemoteService");
//        }
    
        addLine("public interface %s %s {", 
            interfaceName,
            extendedInterfaces.isEmpty() ? ""
                : "extends " + StringUtils.collectionToString(extendedInterfaces, ", ")
        );
        
        increaseIndent();
        addLine();
        
        for (ExecutableElement method : ElementFilter.methodsIn(this.typeElement.getEnclosedElements())) {
            if (method.getAnnotation(XmlTransient.class) != null) {
                continue;
            }
            addLine("public %s %s(", method.getReturnType(), method.getSimpleName());
            increaseIndent();
            
            for (Iterator<? extends VariableElement> i = method.getParameters().iterator(); i.hasNext(); ) {
                VariableElement parameter = i.next();
                addLine("%s %s%s", parameter.asType(), parameter.getSimpleName(), i.hasNext() ? "," : "");
            }

            decreaseIndent();
            
            addLine(")");
            
            if (!method.getThrownTypes().isEmpty()) {
                addLine("throws");
                increaseIndent();
                for (Iterator<? extends TypeMirror> i = method.getThrownTypes().iterator(); i.hasNext(); ) {
                    TypeMirror thrownType = i.next();
                    addLine("%s%s", thrownType, i.hasNext() ? "," : "");
                }
                decreaseIndent();
            }
            
            addLine(";");
            addLine();
        }
        
        decreaseIndent();
        addLine("}");

    }

}
