/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/PosixAccessControlBitMap.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform;

import java.util.HashSet;
import java.util.Set;

import com.netapp.common.platform.FileSecurityDescriptor.AccessControl;

public enum PosixAccessControlBitMap {
    OWNER_READ(AccessControl.OWNER_READ, 0400),
    OWNER_WRITE(AccessControl.OWNER_WRITE, 0200),
    OWNER_EXECUTE(AccessControl.OWNER_EXECUTE, 0100),
    GROUP_READ(AccessControl.GROUP_READ, 0040),
    GROUP_WRITE(AccessControl.GROUP_WRITE, 0020),
    GROUP_EXECUTE(AccessControl.GROUP_EXECUTE, 0010),
    ALL_READ(AccessControl.ALL_READ, 0004),
    ALL_WRITE(AccessControl.ALL_WRITE, 0002),
    ALL_EXECUTE(AccessControl.ALL_EXECUTE, 0001);

    private AccessControl accessControl;
    private int permissionBit;
    
    private PosixAccessControlBitMap(AccessControl accessControl, int permissionBit) {
        this.accessControl = accessControl;
        this.permissionBit = permissionBit;
    }
    
    public static PosixAccessControlBitMap lookup(AccessControl accessControl) {

        for (PosixAccessControlBitMap mapValue : PosixAccessControlBitMap.values()) {
            if (mapValue.accessControl.equals(accessControl)) {
                return mapValue;
            }
        }
        
        // TODO add error message
        throw new IllegalArgumentException();
    }

    public static Set<PosixAccessControlBitMap> findMatches(int permissionBits) {
        
        Set<PosixAccessControlBitMap> matches = new HashSet<PosixAccessControlBitMap>();
        
        for (PosixAccessControlBitMap mapValue : PosixAccessControlBitMap.values()) {
            if ((mapValue.permissionBit & permissionBits) != 0) {
                matches.add(mapValue);
            }
        }
        
        return matches;
    }
    
    public int getPermissionBit() {
        return this.permissionBit;
    }

    public AccessControl getAccessControl() {
        return this.accessControl;
    }
         
}
