/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/freebsd/OperatingSystemImpl.java#1 $
 *
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved. 
 */
package com.netapp.common.platform.freebsd;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellCommand;
import com.netapp.common.execution.ShellExecutor;
import com.netapp.common.execution.ShellExecutorFactory;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.execution.ShellRunnable;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.DeviceAnalyzer;
import com.netapp.common.platform.FileSystemMount;
import com.netapp.common.platform.PartitionTable;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.platform.unix.DFResultConstructor;
import com.netapp.common.util.ConfigProperty;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class OperatingSystemImpl extends com.netapp.common.platform.unix.OperatingSystemImpl {
    private static NALogger logger = NALogger.getLogger(OperatingSystemImpl.class);
    /**
     * pufferfish:/vol/cruise_control_shakedown_oceanic on /mnt/cruise_control/shakedown_oceanic (nfs)
     * /dev/da0s1e on /tmp (ufs, local, soft-updates)
     * pid581@knob-creek:/u on /u (nfs)
     */
    private static final Pattern MOUNT_OUTPUT_PATTERN = Pattern.compile("(pid\\d+@)?(?:(\\S+):)?(\\S+) on (\\S+) \\((\\S+)(?:, ([^\\)]+))?\\).*");
    private static final int MOUNT_OUTPUT_AUTOMOUNT_GROUP = 1;
    private static final int MOUNT_OUTPUT_SERVER_GROUP = 2;
    private static final int MOUNT_OUTPUT_DEVICE_GROUP = 3;
    private static final int MOUNT_OUTPUT_MOUNT_POINT_GROUP = 4;
    private static final int MOUNT_OUTPUT_TYPE_GROUP = 5;
    private static final int MOUNT_OUTPUT_OPTIONS_GROUP = 6;
    /**
     * Values of "os.arch" that should be considered x86
     */
    protected static final ConfigProperty<String> VERSIONS_FREEBSD_ARCH_X86 = ConfigProperty.stringProperty("versions.freebsd.arch.x86");
   
    /**
     * Values of "os.arch" that should be considered x64
     */
    protected static final ConfigProperty<String> VERSIONS_FREEBSD_ARCH_X64 = ConfigProperty.stringProperty("versions.freebsd.arch.x64");

    @Override
    public String getArch() {
        String jvmArch = System.getProperty("os.arch");
        String x86 = VERSIONS_FREEBSD_ARCH_X86.lookup(this.getPlatformConfigBundle());
        String x64 = VERSIONS_FREEBSD_ARCH_X64.lookup(this.getPlatformConfigBundle());
        
        if (jvmArch.matches(x86)) {
            return "x86";
        }
        else if (jvmArch.matches(x64)) {
            return "x64";
        }
        return jvmArch;
    }
    
    @Override
    protected ShellRunnable createDFCommand(List<File> fileList) {
        throw new RuntimeException(
            logger.error(MsgKey.PLATFORM_NO_IMPLEMENTATION));
    }

    @Override
    protected DFResultConstructor createDFResultConstructor() {
        throw new RuntimeException(
            logger.error(MsgKey.PLATFORM_NO_IMPLEMENTATION));
    }

    @Override
    public DeviceAnalyzer createDeviceAnalyzer() {
        throw new RuntimeException(
            logger.error(MsgKey.PLATFORM_NO_IMPLEMENTATION));
    }

    @Override
    public Collection<FileSystemMount> getMountedFilesystems() {
        ShellExecutor executor = ShellExecutorFactory.createShellExecutor();
        ShellCommand shellCommand = new ShellCommand("/sbin/mount");
        ShellResult result = executor.execute(shellCommand, 60 * 1000);
        if (result.isNonZeroExitCode()) {
            throw new PlatformException(
                logger.debug(MsgKey.PLATFORM_MOUNT_TABLE_READ_FAILED, result.allOutAsString()));
        }
        List<FileSystemMount> results = new ArrayList<FileSystemMount>();
        for (String line : result.getAllOut()) {
            Matcher m = MOUNT_OUTPUT_PATTERN.matcher(line);
            if (m.matches()) {
                if (m.group(MOUNT_OUTPUT_AUTOMOUNT_GROUP) != null) {
                    // Skip automount psuedo-mounts
                    continue;
                }
                String server = m.group(MOUNT_OUTPUT_SERVER_GROUP);
                String device = m.group(MOUNT_OUTPUT_DEVICE_GROUP);
                String mountPoint = m.group(MOUNT_OUTPUT_MOUNT_POINT_GROUP);
                String type = m.group(MOUNT_OUTPUT_TYPE_GROUP);
                String options = m.group(MOUNT_OUTPUT_OPTIONS_GROUP);
                options = (options == null) ? "" : options.replaceAll("\\s", "");
                
                results.add(new FileSystemMount(server, device, mountPoint, type, options));
            }
        }
        return results;
    }
    @Override
    public String getLibraryEnvironmentName() {
        return "LD_LIBRARY_PATH";
    }

    @Override
    public PartitionTable readPartitionTable(File device) {
        throw new RuntimeException(
            logger.error(MsgKey.PLATFORM_NO_IMPLEMENTATION));
    }

}