/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/linux/smbios/UuidHeader.java#1 $
 * 
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.linux.smbios;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Formatter;

/**
 * The UUID entry is composed of the following entries and byte counts, in order
 * 
 * <ul>
 * <li>time low - DWORD (4 bytes)
 * <li>time mid - WORD (2 bytes)
 * <li>time high - WORD (2 bytes)
 * <li>clock high - 1 byte
 * <li>clock low - 1 byte
 * <li>node - 6 bytes
 * </ul>
 * 
 * @author josh
 * 
 */
final class UuidHeader {
    private static final char UUID_SEPERATOR = '-';

    private byte[]            timeLow        = new byte[4]; // DWORD, 4 bytes
    private byte[]            timeMid        = new byte[2]; // WORD, 2 bytes
    private byte[]            timeHigh       = new byte[2]; // WORD, 2 bytes
    private byte[]            clockHigh      = new byte[1]; // byte
    private byte[]            clockLow       = new byte[1]; // byte
    private byte[]            node           = new byte[6]; // 6 bytes

    protected UuidHeader() {
    }

    public static UuidHeader read(RandomAccessFile file) throws IOException {
        UuidHeader header = new UuidHeader();

        file.read(header.getTimeLow());
        file.read(header.getTimeMid());
        file.read(header.getTimeHigh());
        file.read(header.getClockHigh());
        file.read(header.getClockLow());
        file.read(header.getNode());

        return header;
    }

    public byte[] getTimeLow() {
        return timeLow;
    }

    protected void setTimeLow(byte[] timeLow) {
        this.timeLow = timeLow;
    }

    public byte[] getTimeMid() {
        return timeMid;
    }

    protected void setTimeMid(byte[] timeMid) {
        this.timeMid = timeMid;
    }

    public byte[] getTimeHigh() {
        return timeHigh;
    }

    protected void setTimeHigh(byte[] timeHigh) {
        this.timeHigh = timeHigh;
    }

    public byte[] getClockHigh() {
        return clockHigh;
    }

    protected void setClockHigh(byte[] clockHigh) {
        this.clockHigh = clockHigh;
    }

    public byte[] getClockLow() {
        return clockLow;
    }

    protected void setClockLow(byte[] clockLow) {
        this.clockLow = clockLow;
    }

    public byte[] getNode() {
        return node;
    }

    protected void setNode(byte[] node) {
        this.node = node;
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();

        builder.append(toHex(getTimeLow()));
        builder.append(UUID_SEPERATOR);
        builder.append(toHex(getTimeMid()));
        builder.append(UUID_SEPERATOR);
        builder.append(toHex(getTimeHigh()));
        builder.append(UUID_SEPERATOR);
        builder.append(toHex(getClockHigh()));
        builder.append(toHex(getClockLow()));
        builder.append(UUID_SEPERATOR);
        builder.append(toHex(getNode()));

        return builder.toString();
    }

    protected String toHex(byte[] buffer) {
        Formatter formatter = new Formatter();

        for (byte b : buffer) {
            formatter.format("%02x", b);
        }

        return formatter.toString();
    }

}
