/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/windows/ServiceControlResult.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.windows;

import static com.netapp.common.MsgKey.*;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.execution.ShellResult;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.PlatformException;

/**
 * Construct a structured result object from the output of
 * the "sc" windows command.  
 * See the <a href=http://msdn2.microsoft.com/en-us/library/ms810435.aspx>MSDN SC Documentation</a>.
 */
class ServiceControlResult {

    private static final NALogger logger = NALogger.getLogger(ServiceControlResult.class);
    
    private ServiceState serviceState;
    
    private static final Pattern STATE_PATTERN = Pattern.compile(
        "\\s*STATE\\s+: \\d+\\s+(STOPPED|PAUSED|RUNNING|STOP_PENDING|PAUSE_PENDING|START_PENDING|CONTINUE_PENDING).*");
    
    private static final Pattern DOES_NOT_EXIST_PATTERN = Pattern.compile("\\[SC\\] EnumQueryServicesStatus:OpenService FAILED 1060:");
    
    ServiceControlResult(ShellResult shellResult, String serviceName) {
        for (String line : shellResult.getStdOut()) {
            Matcher stateMatcher = STATE_PATTERN.matcher(line);
            Matcher doesNotExistMatcher = DOES_NOT_EXIST_PATTERN.matcher(line);
            if (stateMatcher.matches()) {
                String stateString = stateMatcher.group(1);
                this.serviceState = ServiceState.valueOf(stateString);
                break;
            }
            else if (doesNotExistMatcher.find()) {
                this.serviceState = ServiceState.UNDEFINED_SERVICE;
                break;
            }
        }
        
        if (this.serviceState == null) {
            throw new PlatformException(
                logger.error(PLATFORM_WINDOWS_ERROR_QUERYING_SERVICE_STATE, serviceName, shellResult.getAllOut()));
        }
    }
    
    public ServiceState getServiceState() {
        return this.serviceState;
    }

}
