/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/windows/SystemEventNativeImpl.java#1 $
 * 
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.windows;

import java.util.EnumMap;
import java.util.HashMap;
import java.util.Map;

import com.netapp.common.logging.LogLevel;

/**
 * Log to the Windows event logger
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
class SystemEventNativeImpl implements SystemEventNative {
    /** The Windows ERROR type for ReportEvent */
    private static final int EVENTLOG_ERROR_TYPE = 1;
    
    /** The Windows WARNING type for ReportEvent */
    public static final int EVENTLOG_WARNING_TYPE = 2;
    
    /** The Windows INFORMATION type for ReportEvent */
    private static final int EVENTLOG_INFORMATION_TYPE = 4;
    
    /** Map our LogLevel enums to the Windows-defined event types */
    private static final Map<LogLevel, Integer> TYPE_MAP;
    
    /** Map our LogLevel enums to the Category names defined in Messages.mc */
    private static final Map<LogLevel, Integer> CATEGORY_MAP;
    
    /** Map of all open event handles */
    private final Map<String, Integer> handles = new HashMap<String, Integer>();
    
    static {
        System.loadLibrary("com_netapp_common_platform_windows_SystemEventNativeImpl");
        
        TYPE_MAP = new EnumMap<LogLevel, Integer>(LogLevel.class);
        TYPE_MAP.put(LogLevel.TRACE, EVENTLOG_INFORMATION_TYPE);
        TYPE_MAP.put(LogLevel.DEBUG, EVENTLOG_INFORMATION_TYPE);
        TYPE_MAP.put(LogLevel.INFO, EVENTLOG_INFORMATION_TYPE);
        TYPE_MAP.put(LogLevel.PROGRESS, EVENTLOG_INFORMATION_TYPE);
        TYPE_MAP.put(LogLevel.WARN, EVENTLOG_WARNING_TYPE);
        TYPE_MAP.put(LogLevel.ERROR, EVENTLOG_ERROR_TYPE);
        TYPE_MAP.put(LogLevel.FATAL, EVENTLOG_ERROR_TYPE);
        
        CATEGORY_MAP = new EnumMap<LogLevel, Integer>(LogLevel.class);
        CATEGORY_MAP.put(LogLevel.TRACE, 0x01);
        CATEGORY_MAP.put(LogLevel.DEBUG, 0x02);
        CATEGORY_MAP.put(LogLevel.INFO, 0x03);
        CATEGORY_MAP.put(LogLevel.PROGRESS, 0x04);
        CATEGORY_MAP.put(LogLevel.WARN, 0x05);
        CATEGORY_MAP.put(LogLevel.ERROR, 0x06);
        CATEGORY_MAP.put(LogLevel.FATAL, 0x07);
    }
    
    private native void log(int handle, int type, int category, String message);
    private native int registerApplication(String source);
    private native void deregisterApplication(int handle);
    
    /**
     * 
     * @see com.netapp.common.platform.windows.SystemEventNative#emitSystemLog(com.netapp.common.logging.LogLevel, java.lang.String, java.lang.String)
     */
    public void emitSystemLog(LogLevel level, String source, String message) {
        Integer type = TYPE_MAP.get(level);
        Integer category = CATEGORY_MAP.get(level);
        // If one of these keys is null, we don't have a valid mapping for it,
        // just ignore it
        if (type == null || category == null) {
            return;
        }
        
        // Look to see if we already opened up a handle for this
        // source. If we didn't, open one and cache it.
        Integer handle = null;
        synchronized (this.handles) {
            handle = this.handles.get(source);
            if (handle == null) {
                handle = registerApplication(source);
                this.handles.put(source, handle);
            }
        }
        log(handle, type, category, message);
    }
    
    @Override
    public void finalize() throws Throwable {
        // Close all open handles
        for (Integer i : this.handles.values()) {
            if (i != null) {
                deregisterApplication(i);
            }
        }
        super.finalize();
    }
}
