/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/DataSourceFactory.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import static com.netapp.common.util.ConfigProperty.stringProperty;
import static com.netapp.common.util.ConfigProperty.intProperty;

import javax.sql.DataSource;

import org.apache.commons.dbcp.BasicDataSource;

/**
 * Generate a JDBC connection pool from a {@link ConfigBundle}.
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DataSourceFactory {
    public static final String DRIVER_PROPERTY = "jdbc.driver";
    public static final String USER_PROPERTY = "jdbc.user";
    public static final String PASSWORD_PROEPRTY = "jdbc.password";
    public static final String URL_PROPERTY = "jdbc.url";
    public static final String MAX_ACTIVE_PROPERTY = "jdbc.maxActive";
    public static final String INITIAL_SIZE_PROPERTY = "jdbc.initialSize";
    public static final String VALIDATION_QUERY_PROPERTY = "jdbc.validationQuery";
    
    private String prefix;
    private ConfigBundle config;

    private BasicDataSource dataSource;

    public DataSourceFactory() {
        
    }
    
    public DataSourceFactory(ConfigBundle config) {
        this(config, null);
    }

    public DataSourceFactory(ConfigBundle config, String prefix) {
        this.config = config;
        this.prefix = prefix;
    }

    public void setConfig(ConfigBundle config) {
        this.config = config;
    }

    public void setPrefix(String prefix) {
        this.prefix = prefix;
    }

    private void init() {
        String p = (prefix == null) ? "" : (prefix + ".");

        String driver = stringProperty(p + DRIVER_PROPERTY).lookup(config);
        String user = stringProperty(p + USER_PROPERTY).lookup(config);
        String password = stringProperty(p + PASSWORD_PROEPRTY).lookup(config);
        String url = stringProperty(p + URL_PROPERTY).lookup(config);
        Integer maxActive = intProperty(p + MAX_ACTIVE_PROPERTY).lookup(config);
        Integer initialSize = intProperty(p + INITIAL_SIZE_PROPERTY).lookup(config);
        String validationQuery = stringProperty(p + VALIDATION_QUERY_PROPERTY).lookup(config);
        
        if (StringUtils.isEmpty(driver)) {
            throw new IllegalStateException("Driver class not configured");
        }
        if (StringUtils.isEmpty(url)) {
            throw new IllegalStateException("JDBC connection url not configured");
        }

        try {
            Class.forName(driver);
        }
        catch (ClassNotFoundException e) {
            throw new IllegalStateException("Unable to load JDBC driver class " + driver, e);
        }
        
        this.dataSource = new BasicDataSource();
        
        if (maxActive != null && maxActive > 0) {
            this.dataSource.setMaxActive(maxActive);
        }
        if (initialSize != null && initialSize > 0) {
            this.dataSource.setInitialSize(initialSize);
        }
     
        this.dataSource.setDriverClassName(driver);
        this.dataSource.setUsername(user);
        this.dataSource.setPassword(password);
        this.dataSource.setUrl(url);
        
        if (!StringUtils.isEmpty(validationQuery)) {
            this.dataSource.setValidationQuery(validationQuery);
            this.dataSource.setTestOnBorrow(true);
        }
    }

    public DataSource getDataSource() {
        if (this.dataSource == null) {
            init();
        }
        return this.dataSource;
    }

}
