/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/ListenerProxy.java#1 $
 * 
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

/**
 * Maintain a collection of Listeners via WeakReference, and provide a mechanism to
 * execute any callback method against all registered and referenced listeners.
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class ListenerProxy<L> {
    private WeakBag<L> listeners;
    private L proxy;
    
    /**
     * Construct a new Proxy for the specified Listener interface
     * @param clazz
     */
    public ListenerProxy(Class<L> clazz) {
        this.listeners = new WeakBag<L>();
        this.proxy = MulticastProxy.createMulticastProxy(clazz, this.listeners);
    }

    /**
     * Add a new listener to the collection. The listener is maintained by
     * a weak reference, so the caller must maintain a strong reference as
     * long as they want the listener to be called.
     * @param listener
     */
    public void addListener(L listener) {
        listeners.add(listener);
    }
    
    /**
     * Return a proxy interface matching the requested proxy. Executing
     * any callback methods on the returned object will result in the
     * corresponding method being invoked on any active listeners previously
     * registered via {@link #addListener(Object)}
     * @return Proxy interface for the supplied listener
     */
    public L proxy() {
        return this.proxy;
    }
}
