/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/bootstrap/com/netapp/common/util/WildcardTypeImpl.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.lang.reflect.Type;
import java.lang.reflect.WildcardType;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class WildcardTypeImpl implements WildcardType {
    private final Type[] lowerBounds;
    private final Type[] upperBounds;
    private final String string;
    
    WildcardTypeImpl() {
        this.lowerBounds = new Type[] { };
        this.upperBounds = new Type[] {Object.class};
        this.string = "?";
    }
    
    WildcardTypeImpl(Type[] lowerBounds, Type[] upperBounds) {
        this.lowerBounds = lowerBounds;
        this.upperBounds = upperBounds;
        StringBuilder b = new StringBuilder("?");
        if (lowerBounds.length > 0) {
            String prefix = " super ";
            for (Type t : lowerBounds) {
                b.append(prefix);
                if (t instanceof Class) {
                    b.append(((Class)t).getName());
                }
                else {
                    b.append(t);
                }
                prefix = " & ";
            }
        }
        
        else {
            String prefix = " extends ";
            for (Type t : upperBounds) {
                if (Object.class.equals(t)) {
                    continue;
                }
                b.append(prefix);
                if (t instanceof Class) {
                    b.append(((Class)t).getName());
                }
                else {
                    b.append(t);
                }
                prefix = " & ";
            }
        }
        string = b.toString();
    }
    
    /**
     * @see java.lang.reflect.WildcardType#getLowerBounds()
     */
    public Type[] getLowerBounds() {
        return this.lowerBounds;
    }

    /**
     * @see java.lang.reflect.WildcardType#getUpperBounds()
     */
    public Type[] getUpperBounds() {
        return this.upperBounds;
    }

    @Override
    public String toString() {
        return this.string;
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof WildcardType)) {
            return false;
        }
        return this.string.equals(o.toString());
    }
    
    @Override
    public int hashCode() {
        return this.string.hashCode();
    }
}
