/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/xml/DocumentNamespaceContext.java#1 $
 * 
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.xml;

import java.util.Iterator;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;

import org.w3c.dom.Document;

/**
 * Resolves namespace prefixes and URIs against an underlying {@link Document DOM document}.
 * 
 * Useful for executing {@link javax.xml.xpath.XPath XPath} queries against a DOM document.  
 * <p>
 * For example:
 * <blockquote><pre>
 * DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
 * <b>documentBuilderFactory.setNamespaceAware(true);</b>
 * 
 * DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
 * Document document = documentBuilder.parse(...);
 *             
 * XPath xPath = XPathFactory.newInstance().newXPath();
 * xPath.setNamespaceContext(<b>new DocumentNamespaceContext(document)</b>);
 * xPath.evaluate("/ns1:foo/ns2:bar", document);
 * </pre></blockquote>
 * 
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DocumentNamespaceContext implements NamespaceContext {

    private Document document;

    public DocumentNamespaceContext(Document document) {
        this.document = document;
    }
    
    public String getNamespaceURI(String prefix) {
        if (prefix.equals(XMLConstants.DEFAULT_NS_PREFIX)) {
            return this.document.lookupNamespaceURI(null);
        }
        else {
            return this.document.lookupNamespaceURI(prefix);    
        }
    }

    public String getPrefix(String namespaceURI) {
        return this.document.lookupPrefix(namespaceURI);
    }

    public Iterator<String> getPrefixes(String namespaceURI) {
        /*
         * This method is not called by the vast majority of the clients of the NamespaceContext interface.
         * It is not supported by most other NamespaceContext implementations either.
         */
        throw new UnsupportedOperationException();
    }

}
