/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/ProxyZAPIExecutor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi;

import java.io.IOException;
import java.net.UnknownHostException;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;

import netapp.manage.NaProxyServer;
import netapp.manage.NaServer;

/**
 * Executor capable of proxying storage system requests through a DFM
 * server.
 * @author joshb,sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class ProxyZAPIExecutor extends AbstractZAPIExecutor {
    private static final NALogger logger = NALogger.getLogger(ProxyZAPIExecutor.class);
    private String proxyHost;
    
    public ProxyZAPIExecutor(String filer, String proxyServer, String username, String password) throws InvalidTargetException {
        this(filer, proxyServer, DfmZAPIExecutor.DFM_ZAPI_PORT, username, password);
    }

    /**
     * Construct a new ProxyZAPIExecutor 
     * @param filer Target filer for this executor
     * @param proxyServer Hostname of the intermediate DFM server to proxy requests through
     * @param port Port number of the ZAPI interface on the DFM server to connect to
     * @param username Username on the DFM server with api-proxy access
     * @param password Password of the user on the DFM server with api-proxy access
     * @throws InvalidTargetException
     */
    public ProxyZAPIExecutor(String filer, String proxyServer, int port, String username, String password) throws InvalidTargetException {
        super(filer);
        this.proxyHost = proxyServer;
        try {
            NaProxyServer server = new NaProxyServer(proxyServer, filer);
            server.setAdminUser(username, password);
            server.setKeepAliveEnabled(true);
            server.setStyle(NaServer.STYLE_LOGIN_PASSWORD);
            server.setTransportType(NaServer.TRANSPORT_TYPE_HTTPS);        
            server.setServerType(NaServer.SERVER_TYPE_DFM);
            server.setPort((port > 0) ? port : DfmZAPIExecutor.DFM_ZAPI_PORT);
            setNaServer(server);
        }
        catch (UnknownHostException e) {
            throw new InvalidTargetException(
                logger.error(MsgKey.ZAPI_EXECUTOR_UNKNOWN_DFM_PROXY, e, proxyServer, filer));
        }
        try {
            ZAPICommand test = new ZAPICommand("system-get-version");
            this.execute(test);
        }
        catch (ZAPIExecuteException ex) {
            throw new InvalidTargetException(
                logger.error(MsgKey.ZAPI_EXECUTOR_INVALID_PROXY, ex, proxyServer, filer, ex.getLocalizedMessage()));
        }
    }

    public String getProxyHost() {
        return this.proxyHost;
    }
    
    @Override
    public ZAPIResult execute(ZAPICommand command) throws ZAPIExecuteException {
        try {
            return super.execute(command);
        } 
        catch (ZAPIRunnerException e) {
            if (e.getCause() instanceof IOException) {
                throw new ZAPIRunnerException(logger.error(MsgKey.ZAPI_EXECUTOR_DFM_DOWN_PROXY, 
                    e,
                    this.proxyHost,
                    this.filer,
                    e.getCause().getMessage()), 
                    e);
            }
            
            throw e;
        }
    }
    
}
