/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/ZAPIExecutor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi;

import com.netapp.common.logging.NALogger;

/**
 * Interface defining a method of executing raw ZAPI calls.
 * 
 * As a general rule (since Java cannot enforce constructor signatures)
 * any public constructors should have sufficient information to construct a
 * fully working executor without requiring further setters to be called.
 * Constructors should throw {@link InvalidTargetException} if it is able to
 * determine the requested parameters do not construct a working executor.
 * For this reason, deriving from existing executors may require special super
 * constructors that avoid any validation of the completed result.
 * <br>
 * For example, any executor that proxies through another mechanism should validate
 * inside the constructor that there are no name resolution or missing credential issues
 * by executing a test command against the target when complete. This allows applications
 * to catch setup errors all at construction instead of defering unexpeced errors until 
 * the point where the executor is used.
 *
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public interface ZAPIExecutor {
    /**
     * We want the request, error, and response logs to always use this logger so the
     * logging is consistient. All class-specific messages can get logged
     * at their own class.
     * 
     */
    NALogger logger = NALogger.getLogger(ZAPIExecutor.class);

    /**
     * Execute the requested ZAPI Call via this executors mechanism.
     * @param command Command to execute
     * @return ZAPI Result
     * @throws ZAPIExecuteException The remote target returned an error
     * @throws ZAPIRunnerException A error occured processing or executing the ZAPI call
     */
    ZAPIResult execute(ZAPICommand command) throws ZAPIExecuteException;
}