/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/generator/GeneratorFactory.java#1 $
 * 
 * Copyright (c) 2009 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi.generator;

import java.util.Collection;

import com.netapp.common.zapi.parser.Category;
import com.netapp.common.zapi.parser.Field;
import com.netapp.common.zapi.parser.Namespace;
import com.netapp.common.zapi.parser.Type;


public abstract class GeneratorFactory
    <NG extends NamespaceGenerator,
    CG extends NameGenerator<? extends Category>,
    TG extends TypeGenerator<? extends Type>,
    FG extends FieldGenerator<? extends Field>> {
    
    /**
     * Transforms descriptions to the format used by the generated output.
     * Note: the descriptions will have all of the contained original
     * namespace/category/type/field names translated to the
     * the format used in the generated output. 
     */
    private final WordReplacementTransformer descriptionGenerator = new WordReplacementTransformer();

    // allow subclasses to manage optional sets of generation properties.
    public String getProperty(String key) {
        return null;
    }

    public abstract NG getNamespaceGenerator(Namespace namespace);
    
    public abstract CG getCategoryGenerator(Category category);
    
    public abstract TG getTypeGenerator(Type type);
    
    public abstract FG getFieldGenerator(Field field);
    
    public WordReplacementTransformer getDescriptionGenerator() {
        return this.descriptionGenerator;
    }

    /**
     * Adds the word replacements for each namespace/type/field
     * to the description generator.
     */
    public void addWordReplacements(Collection<Namespace> namespaces) {
        for (Namespace namespace : namespaces) {
            namespace.addWordReplacements(this);
        }
    }

}
